@@ -10,7 +10,7 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
+from distutils import spawn
 import os
 import shutil
 import uuid
@@ -19,18 +19,21 @@ import six.moves.urllib.parse as urlparse
 
 from pifpaf import drivers
 from pifpaf.drivers import postgresql
+from pifpaf.drivers import redis
 
 
 class GnocchiDriver(drivers.Driver):
 
     DEFAULT_PORT = 8041
     DEFAULT_PORT_INDEXER = 9541
+    DEFAULT_PORT_COORDINATOR = 9542
 
     def __init__(self, port=DEFAULT_PORT, indexer_port=DEFAULT_PORT_INDEXER,
                  statsd_port=None,
-                 create_legacy_resource_types=False,
                  indexer_url=None,
                  storage_url=None,
+                 coordination_driver="default",
+                 coordination_port=DEFAULT_PORT_COORDINATOR,
                  **kwargs):
         super(GnocchiDriver, self).__init__(**kwargs)
         self.port = port
@@ -38,7 +41,8 @@ class GnocchiDriver(drivers.Driver):
         self.indexer_url = indexer_url
         self.storage_url = storage_url
         self.statsd_port = statsd_port
-        self.create_legacy_resource_types = create_legacy_resource_types
+        self.coordination_driver = coordination_driver
+        self.coordination_port = coordination_port
 
     @classmethod
     def get_parser(cls, parser):
@@ -53,10 +57,15 @@ class GnocchiDriver(drivers.Driver):
                             type=int,
                             default=cls.DEFAULT_PORT_INDEXER,
                             help="port to use for Gnocchi indexer")
-        parser.add_argument("--create-legacy-resource-types",
-                            action='store_true',
-                            default=False,
-                            help="create legacy Ceilometer resource types")
+        parser.add_argument("--coordination-port",
+                            type=int,
+                            default=cls.DEFAULT_PORT_COORDINATOR,
+                            help="port to use for Gnocchi coordination")
+        parser.add_argument("--coordination-driver",
+                            default="default",
+                            choices=["default", "redis"],
+                            nargs="?",
+                            help="Select a coordination driver")
         parser.add_argument("--indexer-url", help="indexer URL to use")
         parser.add_argument("--storage-url", help="storage URL to use")
         return parser
@@ -103,7 +112,7 @@ class GnocchiDriver(drivers.Driver):
             }
         elif storage_driver == "swift":
             storage_config = {
-                "swift_auth_url": "http://%s:%s/%s" % (
+                "swift_authurl": "http://%s:%s%s" % (
                     storage_parsed.hostname,
                     storage_parsed.port,
                     storage_parsed.path,
@@ -130,6 +139,10 @@ class GnocchiDriver(drivers.Driver):
             raise RuntimeError("Storage driver %s is not supported" %
                                storage_driver)
 
+        if self.coordination_driver == "redis":
+            r = self.useFixture(redis.RedisDriver(port=self.coordination_port))
+            storage_config["coordination_url"] = r.url
+
         storage_config_string = "\n".join(
             "%s = %s" % (k, v)
             for k, v in storage_config.items()
@@ -139,12 +152,14 @@ class GnocchiDriver(drivers.Driver):
         with open(conffile, "w") as f:
             f.write("""[DEFAULT]
 debug = %s
+verbose = True
 [storage]
 driver = %s
 %s
 [metricd]
 metric_processing_delay = 1
 metric_cleanup_delay = 1
+workers = 1
 [statsd]
 resource_id = %s
 creator = admin
@@ -157,25 +172,31 @@ url = %s""" % (self.debug,
                statsd_resource_id,
                self.indexer_url))
 
-        gnocchi_upgrade = ["gnocchi-upgrade", "--config-file=%s" % conffile]
-        if self.create_legacy_resource_types:
-            gnocchi_upgrade.append("--create-legacy-resource-types")
-        self._exec(gnocchi_upgrade)
+        self._exec(["gnocchi-upgrade", "--config-file=%s" % conffile])
 
         c, _ = self._exec(["gnocchi-metricd", "--config-file=%s" % conffile],
                           wait_for_line="metrics wait to be processed")
-        self.addCleanup(self._kill, c.pid)
+        self.addCleanup(self._kill, c)
 
         c, _ = self._exec(["gnocchi-statsd", "--config-file=%s" % conffile],
                           wait_for_line=("(Resource .* already exists"
                                          "|Created resource )"))
-        self.addCleanup(self._kill, c.pid)
+        self.addCleanup(self._kill, c)
 
-        c, _ = self._exec(
-            ["gnocchi-api", "--port", str(self.port),
-             "--", "--config-file=%s" % conffile],
-            wait_for_line="Available at http://")
-        self.addCleanup(self._kill, c.pid)
+        c, _ = self._exec([
+            "uwsgi",
+            "--http", "localhost:%d" % self.port,
+            "--wsgi-file", spawn.find_executable("gnocchi-api"),
+            "--master",
+            "--die-on-term",
+            "--lazy-apps",
+            "--no-orphans",
+            "--enable-threads",
+            "--chdir", self.tempdir,
+            "--add-header", "Connection: close",
+            "--pyargv", "--config-file=%s" % conffile,
+        ], wait_for_line="WSGI app 0 \(mountpoint=''\) ready")
+        self.addCleanup(self._kill, c)
 
         self.http_url = "http://localhost:%d" % self.port
 
